/**
 * NNPDF tutorial: How to compute the photon PDF central value 
 * and 1-sigma error band in C++, using LHAPDF interface.
 *
 * Compile with:
 * g++ tutorial.cxx -o tutorial_cxx -I $(shell lhapdf-config --incdir)
 * -L $(shell lhapdf-config --libdir) -lLHAPDF
 * 
 * Author: Stefano Carrazza for The NNPDF Collaboration - 2013
 */

#include <iostream>
#include <cmath>
#include "LHAPDF/LHAPDF.h"
using namespace std;

#define NxPoints 50

// Computes the standard deviation
double ComputeStdDev(int n, double *x, double cv)
{
  double sum = 0;
  for (int i = 0; i < n; i++)
    sum += pow(x[i] - cv, 2.0);

  sum /= n-1;
  return sqrt(sum);
}

int main()
{ 
  // Declaring variables
  const double Q = pow(2.0, 0.5);
  const double xmin = log(1e-5);
  const double xmax = log(1.0);
  const double delta = (xmax - xmin)/NxPoints;
    
  // Loading the NNPDF set
  LHAPDF::initPDFSet("NNPDF23_nlo_as_0119_qed.LHgrid");
  const int NumberReplicas = LHAPDF::numberPDF();

  // Getting the central values
  for (int i = 0; i < NxPoints; i++)
    {
      LHAPDF::initPDF(0); // <- Replica 0 == Central Value (AVG)
      const double x = exp(xmin + i*delta); 
      const double xgCV = LHAPDF::xfxphoton(x,Q,7);      

      // Computing 1-sigma error band
      double ErrorBand[NumberReplicas];
      for (int n = 1; n <= NumberReplicas; n++)
	{
	  LHAPDF::initPDF(n);
	  ErrorBand[n-1] = LHAPDF::xfxphoton(x,Q,7);
	}
      const double xgError = ComputeStdDev(NumberReplicas,ErrorBand,xgCV);

      // Printing results
      cout.precision(5);
      cout << scientific << "x = " << x << "\txphoton(x,Q) = (" << xgCV
	   << "\t+/-\t" << xgError << ")" << endl;
    }

  return 0;
}
